import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.net.URL;

/**
   This is the main user interface for the html version of GameBoyEmu,
   it creates the main window which it shows and the DebugWindows which remain
   hidden until selected

   @author David Winchurch.
   @version 11/03/2001i
**/

public class GameBoyEmuHtml extends Applet implements ActionListener, KeyListener
{
   //instance of GameBoyCPU. This is the virtual processor required to decypher the rom
   private GameBoyCPU CPU;

   //A boolean array used to store the state of a key press
   private boolean[] KEYS_PRESSED={false,false,false,false,false,false,false,false};

   //An array used to set which key combinations do what allowing for configurable keys
   private int[] KEYS={KeyEvent.VK_UP, KeyEvent.VK_DOWN, KeyEvent.VK_LEFT, KeyEvent.VK_RIGHT, KeyEvent.VK_X, KeyEvent.VK_Z, KeyEvent.VK_ENTER, KeyEvent.VK_SHIFT, KeyEvent.VK_SPACE};

   //Integer used to store the video multiplier value. The display is scaled by this factor
   private int vidSize=1;

   //The image that will be drawn when the program is loaded
   private Image splashScreen, gameboy;

   //Panel to keep the buttons in
   private Panel buttonPanel=new Panel(new GridLayout(2,3));

   //Buttons
   private Button vidX1=new Button("X1 Video Mode"),
                  vidX2=new Button("X2 Video Mode"),
                  loadRom=new Button("Load Rom"),
                  paletteEditor=new Button("Edit B&W Palette"),
                  keyConfig=new Button("Configure Keys");

   //A dialog box that will contain the roms that can be loaded
   private Dialog romSelector;

   //The buttons for the rom selector dialog will be stored here
   private Panel romSelectButtonPanel=new Panel(new GridLayout(1, 2, 3, 3));

   //A list that will contain strings representing all the roms that can be loaded
   private List roms=new List(10);

   //The buttons for the rom selector
   private Button romSelectOk=new Button("OK"),
                  romSelectCancel=new Button("Cancel");

   //This array will store all of the rom names
   private String[] romStrings;

   //Unchangeable values for my reference so I know which position in the keys boolean
   //array to set to achieve the required changes
   private final int DIK_UP          = 0;
   private final int DIK_DOWN        = 1;
   private final int DIK_LEFT        = 2;
   private final int DIK_RIGHT       = 3;
   private final int DIK_A           = 4;
   private final int DIK_B           = 5;
   private final int DIK_START       = 6;
   private final int DIK_SELECT      = 7;
   private final int DIK_ALL_BUTTONS = 8;

   /*
      Constructor.
   */

   public GameBoyEmuHtml()
   {

   }

   /**
      This is the main procedure called when GameBoyEmu is run by a web browser.
   **/

   public void init()
   {
      //Sets up the applet
      setLayout(new BorderLayout());
      setBackground(Color.white);
      addKeyListener(this);

      loadRomNames();

      //Loads all the images used for the spinning cog animation on the file loading DebugWindow
      MediaTracker imageTracker = new MediaTracker(this);
      Image[] loadAnim=new Image[9];
      for(int i=0; i<9; i++)
      {
         //loads all the images into a MediaTracker
         loadAnim[i] = getImage(getDocumentBase(), ("./images/cog"+Integer.toString(i+1)+".gif")); imageTracker.addImage(loadAnim[i] ,i);
      }
      //loads the splash screen image
      splashScreen = getImage(getDocumentBase(), "./images/screen.jpg");
      imageTracker.addImage(splashScreen ,9);
      //loads the gameboy image
      gameboy=getImage(getDocumentBase(), "./images/gameboy.gif");
      imageTracker.addImage(gameboy ,10);
      //Waits untill all the images have loaded
      try {imageTracker.waitForAll();}
      catch(InterruptedException e){}

      //Creates a new GameBoyCPU
      CPU=new GameBoyCPU(KEYS_PRESSED, loadAnim);

      //Gets the required handle on the GameBoyVideo instance created in GameBoyCPU
      CPU.getVideo().setVidMult(vidSize);
      //CPU.getVideo().addKeyListener(this);

      //The palette config menuitem should only be selectable if a rom has been loadaed
      paletteEditor.setEnabled(false);

      //Adds action listeners
      vidX1.addActionListener(this);
      vidX2.addActionListener(this);
      loadRom.addActionListener(this);
      keyConfig.addActionListener(this);
      paletteEditor.addActionListener(this);
      romSelectOk.addActionListener(this);
      romSelectCancel.addActionListener(this);

      //Rom selector dialog box button setup
      buttonPanel.add(loadRom);
      buttonPanel.add(keyConfig);
      buttonPanel.add(paletteEditor);
      buttonPanel.add(vidX1);
      buttonPanel.add(vidX2);
      buttonPanel.add(new Label("David Winchurch"));

      //Rom selector dialog box setup
      romSelector=new Dialog(new Frame(), "Select Rom", true);
      romSelector.setResizable(false);
      romSelector.setLayout(new BorderLayout());
      romSelector.setLocation(300 ,200);

      //Sets up the rom selector button panel
      romSelectButtonPanel.add(romSelectOk);
      romSelectButtonPanel.add(romSelectCancel);

      //Sets up the layout of the rom selector
      romSelector.add(roms, BorderLayout.CENTER);
      romSelector.add(romSelectButtonPanel, BorderLayout.SOUTH);
      romSelector.pack();

      //Adds the video and the buttons to the panel
      add(CPU.getVideo(), BorderLayout.CENTER);
      add(buttonPanel, BorderLayout.SOUTH);

      //Makes this the object in focus so that the applet will register keys
      requestFocus();
   }

   /**
      Run when the applet is closed
   **/

   public synchronized void stop()
   {
      CPU.stopThread();
      CPU=null;
      System.gc();
   }

   /**
      Overwritten paint method. Ensures that the correct image is drawn
      when the program is obscured then revealed

      @param g the specified context to use for updating.
   **/

   public void paint(Graphics g)
   {
      //Draws the splash screen
      if(vidSize==1)
         CPU.getVideo().getGraphics().drawImage(splashScreen,0,0,320,288,this);
   }

   /**
      Loads the paths and filenames of all the available roms
   **/

   public void loadRomNames()
   {
      try
      {
         //Creates a new BurreredReader from a file on the hard disk. Throws an error if the file is not present
         DataInputStream romStringReader=new DataInputStream(new URL(getDocumentBase(), "romnames.txt").openStream());

         //Used to track the number of lines in the text file
         int romStringReaderSize=0;

         //Loops until the end of the file
         while (romStringReader.readLine()!=null) romStringReaderSize++;

         //Stores the rom file names
         romStrings=new String[romStringReaderSize];

         //Re opens the file at the beggining
         romStringReader=new DataInputStream(new URL(getDocumentBase(), "romnames.txt").openStream());

         for(int i=0; i<romStringReaderSize; i++)
         {
            String romNameString=romStringReader.readLine();

            //loops through the string checking each character at a time to see if it's an '='
            for(int j=0; j<romNameString.length(); j++)
            {
               if(!romNameString.equals(""))
               {
                  //once it finds the required character these if statements find out if the subtring before
                  //the '=' is part of the stored user information
                  if(romNameString.charAt(j)=='=')
                  {
                     romStrings[roms.getItemCount()]=romNameString.substring(0, j);
                     roms.add(romNameString.substring(j+1, romNameString.length()));
                  }
               }
            }
         }

         romStringReader.close();
      }
      catch(IOException ex){}
   }

   public void loadRom(String romName)
   {
      if(vidSize==1)
         CPU.getVideo().getGraphics().drawImage(splashScreen,0,0,320,288,this);

      //If the loading of a rom was succesful update the DebugWindows and reset the virtual CPU
      if(CPU.loadRom(getDocumentBase() ,"./roms/" + romName))
      {
         //The palete editor menuitem should only be selectable of the rom is not a color rom
         if(CPU.getMem(0x143)>=0x80)
            paletteEditor.setEnabled(false);
         else
            paletteEditor.setEnabled(true);

         CPU.fullReset();
         CPU.startThread();
      }
      requestFocus();
   }

   /**
      This is the ActionEvent handler for GameBoyEmu. It accepts events generated
      by any component with an ActionListener added.

      @param event The event that was generated.
   **/

   public void actionPerformed(ActionEvent event)
   {
      //gets the object that generated the ActionEvent for identification purposes
      Object source=event.getSource();

      //The if statements Discover which component generated the ActionEvent and runs the appropriate methods
      //Updates the variables and frame to accept the new display size
      if(source==loadRom)
      {
         CPU.stopThread();
         romSelector.setVisible(true);
      }
      //Opens the key Configuration window
      else if(source==keyConfig) {KeyConfigurator tempKeyConfig = new KeyConfigurator(new Frame(), KEYS, gameboy);}
      //Opens the palette editor window
      else if(source==paletteEditor) {PaletteEditor tempPalEdit = new PaletteEditor(new Frame(), CPU);}
      else if(source==romSelectOk) {romSelector.setVisible(false); loadRom(romStrings[roms.getSelectedIndex()]);}
      else if(source==romSelectCancel) {romSelector.setVisible(false); CPU.startThread();}
      else if(source==vidX1 && vidSize==1) {repaint();}
      else if(source==vidX1 && vidSize!=1) {vidSize=1; CPU.getVideo().setVidMult(1); repaint(50);}
      else if(source==vidX2 && vidSize!=2) {vidSize=2; CPU.getVideo().setVidMult(2);}

      //Makes this the object in focus so that the applet will register keys
      requestFocus();
   }

   /**
      This is the key pressed event handler for GameBoyEmu. It accepts KeyEvents generated
      by pressing a key when any component with a KeyListener has focus

      @param k The KeyEvent that was generated.
   **/

   public void keyPressed(KeyEvent k)
   {
      //gets the integer corresponding to the pressed key
      int keypress=k.getKeyCode();

      //checks to see if the correct keys have been pressed and sets the corresponding boolean to true
      if (KEYS[DIK_UP]==keypress) {KEYS_PRESSED[DIK_UP]=true;}
      else if (KEYS[DIK_DOWN]==keypress) {KEYS_PRESSED[DIK_DOWN]=true;}
      else if (KEYS[DIK_LEFT]==keypress) {KEYS_PRESSED[DIK_LEFT]=true;}
      else if (KEYS[DIK_RIGHT]==keypress) {KEYS_PRESSED[DIK_RIGHT]=true;}

      else if (KEYS[DIK_A]==keypress) {KEYS_PRESSED[DIK_A]=true;}
      else if (KEYS[DIK_B]==keypress) {KEYS_PRESSED[DIK_B]=true;}
      else if (KEYS[DIK_START]==keypress) {KEYS_PRESSED[DIK_START]=true;}
      else if (KEYS[DIK_SELECT]==keypress) {KEYS_PRESSED[DIK_SELECT]=true;}

      else if (KEYS[DIK_ALL_BUTTONS]==keypress) {KEYS_PRESSED[DIK_A]=true; KEYS_PRESSED[DIK_B]=true; KEYS_PRESSED[DIK_START]=true; KEYS_PRESSED[DIK_SELECT]=true;}
   }

   /**
      This is the key released event handler for GameBoyEmu. It accepts KeyEvents generated
      by releasing a key when any component with a KeyListener has focus

      @param k The KeyEvent that was generated.
   **/

   public void keyReleased(KeyEvent k)
   {
      //gets the integer corresponding to the released key
      int keypress=k.getKeyCode();

      //checks to see if the correct keys have been released and sets the corresponding boolean to false
      if (KEYS[DIK_UP]==keypress) KEYS_PRESSED[DIK_UP]=false;
      else if (KEYS[DIK_DOWN]==keypress) KEYS_PRESSED[DIK_DOWN]=false;
      else if (KEYS[DIK_LEFT]==keypress) KEYS_PRESSED[DIK_LEFT]=false;
      else if (KEYS[DIK_RIGHT]==keypress) KEYS_PRESSED[DIK_RIGHT]=false;

      else if (KEYS[DIK_A]==keypress) KEYS_PRESSED[DIK_A]=false;
      else if (KEYS[DIK_B]==keypress) KEYS_PRESSED[DIK_B]=false;
      else if (KEYS[DIK_START]==keypress) KEYS_PRESSED[DIK_START]=false;
      else if (KEYS[DIK_SELECT]==keypress) KEYS_PRESSED[DIK_SELECT]=false;

      else if (KEYS[DIK_ALL_BUTTONS]==keypress) {KEYS_PRESSED[DIK_A]=false; KEYS_PRESSED[DIK_B]=false; KEYS_PRESSED[DIK_START]=false; KEYS_PRESSED[DIK_SELECT]=false;}

   }

   /*
      This is the key typed event handler for GameBoyEmu. It is here because java requires you
      to implement it for KeyListeners to work. Dos nothing in this program

      @param k The KeyEvent that was generated.
   */

   public void keyTyped(KeyEvent k) {}
}
